"use strict";

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

let fs = require("fs");
let path = require("path");
let { prepackSources } = require("../lib/prepack-node.js");
let babel = require("babel-core");
let React = require("react");
let ReactDOM = require("react-dom");
let ReactDOMServer = require("react-dom/server");
let PropTypes = require("prop-types");
let ReactRelay = require("react-relay");
let ReactTestRenderer = require("react-test-renderer");
let { mergeAdacentJSONTextNodes } = require("../lib/utils/json.js");
/* eslint-disable no-undef */
let { expect, describe, it } = global;

function cxShim(...args) {
  let classNames = [];
  for (let arg of args) {
    if (typeof arg === "string") {
      classNames.push(arg);
    } else if (typeof arg === "object" && arg !== null) {
      let keys = Object.keys(arg);
      for (let key of keys) {
        if (arg[key]) {
          classNames.push(key);
        }
      }
    }
  }
  return classNames.join(" ");
}

// assign for tests that use the cx() global
global.cx = cxShim;

function getDataFile(directory, name) {
  let reactTestRoot = path.join(__dirname, "../test/react/");
  let data = fs.readFileSync(path.join(reactTestRoot, directory, name)).toString();
  return data;
}

function MockURI(url) {
  this.url = url;
}

MockURI.prototype.addQueryData = function () {
  this.url += "&queryData";
  return this;
};

MockURI.prototype.makeString = function () {
  return this.url;
};

function runTestSuite(outputJsx, shouldTranspileSource) {
  let expectReconcilerFatalError = (() => {
    var _ref = _asyncToGenerator(function* (func) {
      checkForReconcilerFatalError = true;
      try {
        yield func();
      } catch (e) {
        expect(e.__isReconcilerFatalError).toBe(true);
        expect(e.message).toMatchSnapshot();
      } finally {
        checkForReconcilerFatalError = false;
      }
    });

    return function expectReconcilerFatalError(_x) {
      return _ref.apply(this, arguments);
    };
  })();

  let expectPartialKeyOrRefError = (() => {
    var _ref2 = _asyncToGenerator(function* (func) {
      checkForPartialKeyOrRefError = true;
      try {
        yield func();
      } catch (e) {
        expect(e.__isReconcilerFatalError).toBe(true);
        expect(e.message).toMatchSnapshot();
      } finally {
        checkForPartialKeyOrRefError = false;
      }
    });

    return function expectPartialKeyOrRefError(_x2) {
      return _ref2.apply(this, arguments);
    };
  })();

  let runTest = (() => {
    var _ref3 = _asyncToGenerator(function* (directory, name, firstRenderOnly = false, data) {
      let source = fs.readFileSync(path.join(reactTestRoot, directory, name)).toString();
      if (shouldTranspileSource) {
        source = transpileSource(source);
      }
      let { compiledSource, statistics } = compileSourceWithPrepack(source);

      expect(statistics).toMatchSnapshot();
      let A = runSource(source);
      let B = runSource(compiledSource);

      expect(typeof A).toBe(typeof B);
      if (typeof A !== "function") {
        // Test without exports just verifies that the file compiles.
        return;
      }

      let config = {
        createNodeMock(x) {
          return x;
        }
      };
      let rendererA = ReactTestRenderer.create(null, config);
      let rendererB = ReactTestRenderer.create(null, config);
      if (A == null || B == null) {
        throw new Error("React test runner issue");
      }
      // Use the original version of the test in case transforming messes it up.
      let { getTrials: getTrialsA, independent } = A;
      let { getTrials: getTrialsB } = B;
      // Run tests that assert the rendered output matches.
      let resultA = getTrialsA(rendererA, A, data);
      let resultB = independent ? getTrialsB(rendererB, B, data) : getTrialsA(rendererB, B, data);

      // The test has returned many values for us to check
      for (let i = 0; i < resultA.length; i++) {
        let [nameA, valueA] = resultA[i];
        let [nameB, valueB] = resultB[i];
        if (typeof valueA === "string" && typeof valueB === "string") {
          expect(valueA).toBe(valueB);
        } else {
          expect(mergeAdacentJSONTextNodes(valueB, firstRenderOnly)).toEqual(mergeAdacentJSONTextNodes(valueA, firstRenderOnly));
        }
        expect(nameB).toEqual(nameA);
      }
    });

    return function runTest(_x3, _x4) {
      return _ref3.apply(this, arguments);
    };
  })();

  let stubReactRelay = (() => {
    var _ref4 = _asyncToGenerator(function* (f) {
      let oldReactRelay = ReactRelay;
      ReactRelay = {
        QueryRenderer(props) {
          return props.render({ props: {}, error: null });
        },
        createFragmentContainer() {
          return null;
        },
        graphql() {
          return null;
        }
      };
      try {
        yield f();
      } finally {
        ReactRelay = oldReactRelay;
      }
    });

    return function stubReactRelay(_x5) {
      return _ref4.apply(this, arguments);
    };
  })();

  // Jest tests


  let checkForReconcilerFatalError = false;
  let checkForPartialKeyOrRefError = false;
  let errorsCaptured = [];
  let reactTestRoot = path.join(__dirname, "../test/react/");
  let prepackOptions = {
    errorHandler: diag => {
      errorsCaptured.push(diag);
      if (diag.severity !== "Warning" && diag.severity !== "Information") {
        if (diag.errorCode === "PP0025" && !checkForPartialKeyOrRefError) {
          // recover from `unable to evaluate "key" and "ref" on a ReactElement
          return "Recover";
        }
        return "Fail";
      }
      return "Recover";
    },
    compatibility: "fb-www",
    internalDebug: true,
    serialize: true,
    uniqueSuffix: "",
    maxStackDepth: 100,
    reactEnabled: true,
    reactOutput: outputJsx ? "jsx" : "create-element",
    reactOptimizeNestedFunctions: true,
    inlineExpressions: true,
    invariantLevel: 0,
    stripFlow: true
  };

  function compileSourceWithPrepack(source) {
    let code = `(function(){${source}})()`;
    let serialized;
    errorsCaptured = [];
    try {
      serialized = prepackSources([{ filePath: "", fileContents: code, sourceMapContents: "" }], prepackOptions);
    } catch (e) {
      if (e.__isReconcilerFatalError && (checkForReconcilerFatalError || checkForPartialKeyOrRefError)) {
        throw e;
      }
      errorsCaptured.forEach(error => {
        console.error(error);
      });
      throw e;
    }
    if (serialized == null || serialized.reactStatistics == null) {
      throw new Error("React test runner failed during serialization");
    }
    return {
      compiledSource: serialized.code,
      statistics: serialized.reactStatistics
    };
  }

  function transpileSource(source) {
    return babel.transform(source, {
      presets: ["babel-preset-react"],
      plugins: ["transform-object-rest-spread"]
    }).code;
  }

  function runSource(source) {
    let transformedSource = transpileSource(source);
    /* eslint-disable no-new-func */
    let fn = new Function("require", "module", transformedSource);
    let moduleShim = { exports: null };
    let requireShim = name => {
      switch (name) {
        case "React":
        case "react":
          return React;
        case "react-dom":
        case "ReactDOM":
          return ReactDOM;
        case "react-dom/server":
        case "ReactDOMServer":
          return ReactDOMServer;
        case "PropTypes":
        case "prop-types":
          return PropTypes;
        case "RelayModern":
          return ReactRelay;
        case "cx":
          return cxShim;
        case "FBEnvironment":
          return {};
        case "URI":
          return MockURI;
        default:
          throw new Error(`Unrecognized import: "${name}".`);
      }
    };
    try {
      // $FlowFixMe flow doesn't new Function
      fn(requireShim, moduleShim);
    } catch (e) {
      console.error(transformedSource);
      throw e;
    }
    return moduleShim.exports;
  }

  let originalConsoleError = global.console.error;
  // we don't want React's errors printed, it's too much noise
  let excludeErrorsContaining = ["Nothing was returned from render. This usually means a return statement is missing. Or, to render nothing, return null", "Consider adding an error boundary to your tree to customize error handling behavior.", "Warning:"];
  global.console.error = function (...args) {
    let text = args[0];

    if (typeof text === "string") {
      for (let excludeError of excludeErrorsContaining) {
        if (text.indexOf(excludeError) !== -1) {
          return;
        }
      }
    }
    originalConsoleError.apply(this, args);
  };

  describe(`Test React with ${shouldTranspileSource ? "create-element input" : "JSX input"}, ${outputJsx ? "JSX output" : "create-element output"}`, () => {
    describe("Functional component folding", () => {
      let directory = "functional-components";

      it("Simple", _asyncToGenerator(function* () {
        yield runTest(directory, "simple.js");
      }));

      it("Simple 2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-2.js");
      }));

      it("Simple 3", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-3.js");
      }));

      it("Simple 4", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-4.js");
      }));

      it("Simple 5", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-5.js");
      }));

      it("Simple 6", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-6.js");
      }));

      it("Simple 7", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-7.js");
      }));

      it("Simple 8", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "simple-8.js");
        }));
      }));

      it("Simple 9", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "simple-9.js");
        }));
      }));

      it("Simple 10", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "simple-10.js");
        }));
      }));

      it("Simple 11", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "simple-11.js");
        }));
      }));

      it("Simple 12", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-12.js");
      }));

      // this should intentionally fail
      it("Runtime error", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "runtime-error.js");
        }));
      }));

      it("Simple 13", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "simple-13.js");
        }));
      }));

      it("Simple 14", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-14.js");
      }));

      it("Simple 15", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-15.js");
      }));

      it("Simple 16", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-16.js");
      }));

      it("Simple 17", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-17.js");
      }));

      it("Simple 18", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-18.js");
      }));

      it("Simple 19", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-19.js");
      }));

      it("Simple 20", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-20.js");
      }));

      it("Simple 21", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-21.js");
      }));

      it("Havocing of ReactElements should not result in property assignments", _asyncToGenerator(function* () {
        yield runTest(directory, "react-element-havoc.js");
      }));

      it("__reactCompilerDoNotOptimize", _asyncToGenerator(function* () {
        yield runTest(directory, "do-not-optimize.js");
      }));

      it("Mutations - not-safe 1", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "not-safe.js");
        }));
      }));

      it("Mutations - not-safe 2", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "not-safe2.js");
        }));
      }));

      it("Mutations - safe 1", _asyncToGenerator(function* () {
        yield runTest(directory, "safe.js");
      }));

      it("Mutations - safe 2", _asyncToGenerator(function* () {
        yield runTest(directory, "safe2.js");
      }));

      it("Handle mapped arrays", _asyncToGenerator(function* () {
        yield runTest(directory, "array-map.js");
      }));

      it("Handle mapped arrays 2", _asyncToGenerator(function* () {
        yield runTest(directory, "array-map2.js");
      }));

      it("Simple fragments", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-fragments.js");
      }));

      it("Simple children", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-children.js");
      }));

      it("Simple with new expression", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-new-expression.js");
      }));

      it("Simple refs", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-refs.js");
      }));

      it("16.3 refs", _asyncToGenerator(function* () {
        yield runTest(directory, "refs.js");
      }));

      it("16.3 refs 2", _asyncToGenerator(function* () {
        yield runTest(directory, "refs2.js");
      }));

      it("16.3 refs 3", _asyncToGenerator(function* () {
        yield runTest(directory, "refs3.js");
      }));

      it("defaultProps", _asyncToGenerator(function* () {
        yield runTest(directory, "default-props.js");
      }));

      it("Unsafe spread", _asyncToGenerator(function* () {
        yield expectPartialKeyOrRefError(_asyncToGenerator(function* () {
          yield runTest(directory, "unsafe-spread.js");
        }));
      }));

      it("Simple with abstract props", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-abstract-props.js");
      }));

      it("Simple with unary expressions", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-unary.js");
      }));

      it("Simple with multiple JSX spreads", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread.js");
      }));

      it("Simple with multiple JSX spreads #2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread2.js");
      }));

      it("Simple with multiple JSX spreads #3", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread3.js");
      }));

      it("Simple with multiple JSX spreads #4", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread4.js");
      }));

      it("Simple with multiple JSX spreads #5", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread5.js");
      }));

      it("Simple with multiple JSX spreads #6", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread6.js");
      }));

      it("Simple with multiple JSX spreads #7", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread7.js");
      }));

      it("Simple with multiple JSX spreads #8", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread8.js");
      }));

      it("Simple with multiple JSX spreads #9", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread9.js");
      }));

      it("Simple with multiple JSX spreads #10", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread10.js");
      }));

      it("Simple with multiple JSX spreads #11", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread11.js");
      }));

      it("Simple with multiple JSX spreads #12", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread12.js");
      }));

      it("Simple with multiple JSX spreads #13", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-with-jsx-spread13.js");
      }));

      it("Simple with Object.assign", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-assign.js");
      }));

      it("Simple with Object.assign #2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-assign2.js");
      }));

      it("Simple with Object.assign #3", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-assign3.js");
      }));

      it("Simple with Object.assign #4", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-assign4.js");
      }));

      it("Simple with Object.assign #5", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-assign5.js");
      }));

      it("Circular reference", _asyncToGenerator(function* () {
        yield runTest(directory, "circular-reference.js");
      }));

      it("Conditional", _asyncToGenerator(function* () {
        yield runTest(directory, "conditional.js");
      }));

      it("Key nesting", _asyncToGenerator(function* () {
        yield runTest(directory, "key-nesting.js");
      }));

      it("Key nesting 2", _asyncToGenerator(function* () {
        yield runTest(directory, "key-nesting-2.js");
      }));

      it("Key nesting 3", _asyncToGenerator(function* () {
        yield runTest(directory, "key-nesting-3.js");
      }));

      it("Key change", _asyncToGenerator(function* () {
        yield runTest(directory, "key-change.js");
      }));

      it("Equivalence", _asyncToGenerator(function* () {
        let createElement = React.createElement;
        let count = 0;
        // For this test we want to also check how React.createElement
        // calls occur so we can validate that we are correctly using
        // lazy branched elements. To do this, we override the createElement
        // call and increment a counter for ever call.

        // $FlowFixMe: intentional for this test
        React.createElement = function (type, config) {
          count++;
          return createElement(type, config);
        };
        try {
          yield runTest(directory, "equivalence.js");
        } finally {
          // $FlowFixMe: intentional for this test
          React.createElement = createElement;
        }
        // The non-compiled version has 20 calls, the compiled should have 8 calls
        expect(count).toEqual(28);
      }));

      it("Delete element prop key", _asyncToGenerator(function* () {
        yield runTest(directory, "delete-element-prop-key.js");
      }));

      it("Key change with fragments", _asyncToGenerator(function* () {
        yield runTest(directory, "key-change-fragments.js");
      }));

      it("Key not changing with fragments", _asyncToGenerator(function* () {
        yield runTest(directory, "key-not-change-fragments.js");
      }));

      it("Component type change", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change.js");
      }));

      it("Component type change 2", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change2.js");
      }));

      it("Component type change 3", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change3.js");
      }));

      it("Component type change 4", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change4.js");
      }));

      it("Component type change 5", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change5.js");
      }));

      it("Component type change 6", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change6.js");
      }));

      it("Component type change 7", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change7.js");
      }));

      it("Component type change 8", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change8.js");
      }));

      it("Component type change 9", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change9.js");
      }));

      it("Component type change 10", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change10.js");
      }));

      it("Component type change 11", _asyncToGenerator(function* () {
        yield runTest(directory, "type-change11.js");
      }));

      it("Component type same", _asyncToGenerator(function* () {
        yield runTest(directory, "type-same.js");
      }));

      it("Dynamic props", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-props.js");
      }));

      it("Dynamic context", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-context.js");
      }));

      it("React.cloneElement", _asyncToGenerator(function* () {
        yield runTest(directory, "clone-element.js");
      }));

      it("Return text", _asyncToGenerator(function* () {
        yield runTest(directory, "return-text.js");
      }));

      it("Render array twice", _asyncToGenerator(function* () {
        yield runTest(directory, "array-twice.js");
      }));

      it("Render nested array children", _asyncToGenerator(function* () {
        yield runTest(directory, "nested-array-children.js");
      }));

      it("Return undefined", _asyncToGenerator(function* () {
        yield runTest(directory, "return-undefined.js");
      }));

      it("Null or undefined props", _asyncToGenerator(function* () {
        yield runTest(directory, "null-or-undefined-props.js");
      }));

      it("Event handlers", _asyncToGenerator(function* () {
        yield runTest(directory, "event-handlers.js");
      }));

      it("Class component as root", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root.js");
      }));

      it("Class component as root with multiple render methods", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-render-methods.js");
      }));

      it("Class component as root with props", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-props.js");
      }));

      it("Class component as root with state", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-state.js");
      }));

      it("Class component as root with refs", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-refs.js");
      }));

      it("Class component as root with instance variables", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-instance-vars.js");
      }));

      it("Class component as root with instance variables #2", _asyncToGenerator(function* () {
        yield runTest(directory, "class-root-with-instance-vars-2.js");
      }));

      it("Additional functions closure scope capturing", _asyncToGenerator(function* () {
        yield runTest(directory, "additional-function-regression.js");
      }));

      it("Dynamic ReactElement type", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-type.js");
      }));

      it("Dynamic ReactElement type #2", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-type2.js");
      }));

      it("Dynamic ReactElement type #3", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-type3.js");
      }));

      it("Dynamic ReactElement type #4", _asyncToGenerator(function* () {
        yield runTest(directory, "dynamic-type4.js");
      }));

      it("Lazy branched elements", _asyncToGenerator(function* () {
        let createElement = React.createElement;
        let count = 0;
        // For this test we want to also check how React.createElement
        // calls occur so we can validate that we are correctly using
        // lazy branched elements. To do this, we override the createElement
        // call and increment a counter for ever call.

        // $FlowFixMe: intentional for this test
        React.createElement = function (type, config) {
          count++;
          return createElement(type, config);
        };
        try {
          yield runTest(directory, "lazy-branched-elements.js");
        } finally {
          // $FlowFixMe: intentional for this test
          React.createElement = createElement;
        }
        // The non-compiled version has 4 calls, the compiled should have 4 calls
        expect(count).toEqual(8);
      }));

      it("Lazy branched elements 2", _asyncToGenerator(function* () {
        let createElement = React.createElement;
        let count = 0;
        // For this test we want to also check how React.createElement
        // calls occur so we can validate that we are correctly using
        // lazy branched elements. To do this, we override the createElement
        // call and increment a counter for ever call.

        // $FlowFixMe: intentional for this test
        React.createElement = function (type, config) {
          count++;
          return createElement(type, config);
        };
        try {
          yield runTest(directory, "lazy-branched-elements2.js");
        } finally {
          // $FlowFixMe: intentional for this test
          React.createElement = createElement;
        }
        // The non-compiled version has 4 calls, the compiled should have 3 calls
        // (3 because one of the calls has been removing by inlining)
        expect(count).toEqual(7);
      }));
    });

    describe("Class component folding", () => {
      let directory = "class-components";

      it("Simple", _asyncToGenerator(function* () {
        yield runTest(directory, "simple.js");
      }));

      it("Simple classes", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-classes.js");
      }));

      it("Simple classes #2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-classes-2.js");
      }));

      it("Simple classes #3", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-classes-3.js");
      }));

      it("Simple classes with Array.from", _asyncToGenerator(function* () {
        yield runTest(directory, "array-from.js");
      }));

      it("Simple classes with Array.from 2", _asyncToGenerator(function* () {
        yield runTest(directory, "array-from2.js");
      }));

      it("Inheritance chaining", _asyncToGenerator(function* () {
        yield runTest(directory, "inheritance-chain.js");
      }));

      it("Classes with state", _asyncToGenerator(function* () {
        yield runTest(directory, "classes-with-state.js");
      }));

      it("Complex class components folding into functional root component", _asyncToGenerator(function* () {
        yield runTest(directory, "complex-class-into-functional-root.js");
      }));

      it("Complex class components folding into functional root component #2", _asyncToGenerator(function* () {
        yield runTest(directory, "complex-class-into-functional-root2.js");
      }));

      it("Complex class components folding into functional root component #3", _asyncToGenerator(function* () {
        yield runTest(directory, "complex-class-into-functional-root3.js");
      }));

      it("Complex class components folding into functional root component #4", _asyncToGenerator(function* () {
        yield runTest(directory, "complex-class-into-functional-root4.js");
      }));

      it("Complex class components folding into functional root component #5", _asyncToGenerator(function* () {
        yield runTest(directory, "complex-class-into-functional-root5.js");
      }));
    });

    describe("Factory class component folding", () => {
      let directory = "factory-components";

      it("Simple factory classes", _asyncToGenerator(function* () {
        yield runTest(directory, "simple.js");
      }));

      it("Simple factory classes 2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple2.js");
      }));
    });

    describe("Render props", () => {
      let directory = "render-props";

      it("Relay QueryRenderer", _asyncToGenerator(function* () {
        yield runTest(directory, "relay-query-renderer.js");
      }));

      it("Relay QueryRenderer 2", _asyncToGenerator(function* () {
        yield runTest(directory, "relay-query-renderer2.js");
      }));

      it("Relay QueryRenderer 3", _asyncToGenerator(function* () {
        yield runTest(directory, "relay-query-renderer3.js");
      }));

      it("React Context", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context.js");
      }));

      it("React Context 2", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context2.js");
      }));

      it("React Context 3", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context3.js");
      }));

      it("React Context 4", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context4.js");
      }));

      it("React Context 5", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context5.js");
      }));

      it("React Context 6", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context6.js");
      }));

      it("React Context 7", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context7.js");
      }));

      it("React Context from root tree", _asyncToGenerator(function* () {
        yield runTest(directory, "react-root-context.js");
      }));

      it("React Context from root tree 2", _asyncToGenerator(function* () {
        yield runTest(directory, "react-root-context2.js");
      }));

      it("React Context from root tree 3", _asyncToGenerator(function* () {
        yield runTest(directory, "react-root-context3.js");
      }));

      it("React Context from root tree 4", _asyncToGenerator(function* () {
        yield runTest(directory, "react-root-context4.js");
      }));
    });

    describe("First render only", () => {
      let directory = "first-render-only";

      it("Simple", _asyncToGenerator(function* () {
        yield runTest(directory, "simple.js", true);
      }));

      it("Simple #2", _asyncToGenerator(function* () {
        yield runTest(directory, "simple-2.js", true);
      }));

      it("componentWillMount", _asyncToGenerator(function* () {
        yield runTest(directory, "will-mount.js", true);
      }));

      it("getDerivedStateFromProps", _asyncToGenerator(function* () {
        yield runTest(directory, "get-derived-state-from-props.js", true);
      }));

      it("getDerivedStateFromProps 2", _asyncToGenerator(function* () {
        yield runTest(directory, "get-derived-state-from-props2.js", true);
      }));

      it("getDerivedStateFromProps 3", _asyncToGenerator(function* () {
        yield runTest(directory, "get-derived-state-from-props3.js", true);
      }));

      it("getDerivedStateFromProps 4", _asyncToGenerator(function* () {
        yield runTest(directory, "get-derived-state-from-props4.js", true);
      }));

      it("getDerivedStateFromProps 5", _asyncToGenerator(function* () {
        yield runTest(directory, "get-derived-state-from-props5.js", true);
      }));

      it("React Context", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context.js");
      }));

      it("React Context 2", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context2.js");
      }));

      it("React Context 3", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context3.js");
      }));

      it("React Context 4", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context4.js");
      }));

      it("React Context 5", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context5.js");
      }));

      it("React Context 6", _asyncToGenerator(function* () {
        yield runTest(directory, "react-context6.js");
      }));

      it.skip("Replace this in callbacks", _asyncToGenerator(function* () {
        yield runTest(directory, "replace-this-in-callbacks.js");
      }));

      it("Replace this in callbacks 2", _asyncToGenerator(function* () {
        yield runTest(directory, "replace-this-in-callbacks2.js");
      }));

      it("Replace this in callbacks 3", _asyncToGenerator(function* () {
        yield runTest(directory, "replace-this-in-callbacks3.js");
      }));
    });

    describe("fb-www mocks", () => {
      let directory = "mocks";

      it("fb-www", _asyncToGenerator(function* () {
        yield stubReactRelay(_asyncToGenerator(function* () {
          yield runTest(directory, "fb1.js");
        }));
      }));

      it("fb-www 2", _asyncToGenerator(function* () {
        yield runTest(directory, "fb2.js");
      }));

      it("fb-www 3", _asyncToGenerator(function* () {
        yield stubReactRelay(_asyncToGenerator(function* () {
          yield runTest(directory, "fb3.js");
        }));
      }));

      it("fb-www 4", _asyncToGenerator(function* () {
        yield stubReactRelay(_asyncToGenerator(function* () {
          yield runTest(directory, "fb4.js");
        }));
      }));

      it("fb-www 5", _asyncToGenerator(function* () {
        yield runTest(directory, "fb5.js");
      }));

      it("fb-www 6", _asyncToGenerator(function* () {
        yield runTest(directory, "fb6.js");
      }));

      it("fb-www 7", _asyncToGenerator(function* () {
        yield runTest(directory, "fb7.js");
      }));

      it("fb-www 8", _asyncToGenerator(function* () {
        yield runTest(directory, "fb8.js");
      }));

      it("fb-www 9", _asyncToGenerator(function* () {
        yield runTest(directory, "fb9.js");
      }));

      it("fb-www 10", _asyncToGenerator(function* () {
        yield runTest(directory, "fb10.js");
      }));

      it("fb-www 11", _asyncToGenerator(function* () {
        yield runTest(directory, "fb11.js");
      }));

      it("fb-www 12", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "fb12.js");
        }));
      }));

      it("fb-www 13", _asyncToGenerator(function* () {
        yield runTest(directory, "fb13.js");
      }));

      it("fb-www 14", _asyncToGenerator(function* () {
        yield runTest(directory, "fb14.js");
      }));

      it("fb-www 15", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "fb15.js");
        }));
      }));

      it("fb-www 16", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "fb16.js");
        }));
      }));

      it("fb-www 17", _asyncToGenerator(function* () {
        yield runTest(directory, "fb17.js");
      }));

      // Test fails for two reasons:
      // - "uri.foo" on abstract string does not exist
      // - unused.bar() does not exist (even if in try/catch)
      it("fb-www 18", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "fb18.js");
        }));
      }));

      it("fb-www 19", _asyncToGenerator(function* () {
        yield expectReconcilerFatalError(_asyncToGenerator(function* () {
          yield runTest(directory, "fb19.js");
        }));
      }));

      it("fb-www 20", _asyncToGenerator(function* () {
        yield runTest(directory, "fb20.js");
      }));

      it("fb-www 21", _asyncToGenerator(function* () {
        yield runTest(directory, "fb21.js");
      }));

      it("fb-www 22", _asyncToGenerator(function* () {
        yield runTest(directory, "fb22.js");
      }));

      it("fb-www 23", _asyncToGenerator(function* () {
        yield runTest(directory, "fb23.js");
      }));

      it("repl example", _asyncToGenerator(function* () {
        yield runTest(directory, "repl-example.js");
      }));

      it("Hacker News app", _asyncToGenerator(function* () {
        let data = JSON.parse(getDataFile(directory, "hacker-news.json"));
        yield runTest(directory, "hacker-news.js", false, data);
      }));

      it("Function bind", _asyncToGenerator(function* () {
        yield runTest(directory, "function-bind.js");
      }));
    });

    describe("react-dom server rendering", () => {
      let directory = "server-rendering";

      it("Hacker News app", _asyncToGenerator(function* () {
        let data = JSON.parse(getDataFile(directory, "hacker-news.json"));
        yield runTest(directory, "hacker-news.js", false, data);
      }));
    });

    describe("react-dom", () => {
      let directory = "react-dom";

      it("createPortal", _asyncToGenerator(function* () {
        yield runTest(directory, "create-portal.js", false);
      }));
    });
  });
}

// pre non-transpiled
runTestSuite(true, false);
runTestSuite(false, false);
// pre transpiled
runTestSuite(true, true);
runTestSuite(false, true);
//# sourceMappingURL=test-react.js.map